/*-------------------------------------------------------------------*/
/* CBLS.C         Pasi Frnti                                        */
/*                                                                   */
/* Console interface for Local Search (LS) algorithm				 */
/*                                                                   */
/*-------------------------------------------------------------------*/

#define ProgName       "CBLS"
#define VersionNumber  "Version 0.06"
#define LastUpdated    "5.3.99"

/* ----------------------------------------------------------------- */

#define  FACTFILE  "cbls.fac"

#include "parametr.c"
#include "ls.h"
#include "cb.h"
#include "file.h"
#include "interfc.h"
#include "memctrl.h"
#include "sortcb.h"

/*--------------------------- Constants -----------------------------*/

#define  MAXFILENAME    64

/*----------------------------- Macros ------------------------------*/

#define  InitialSolutionGiven	(InitCBName[0] != 0x00)



/*======================== Print routines ===========================*/


void PrintInfo(void)
{
  printf("\n%s   %s   %s\n", ProgName, VersionNumber, LastUpdated);
  printf("LS - Local Search algorithm using Swapping Techniques.\n");
  printf("Usage: %s [%coption] <training set> <codebook>\n", ProgName, OPTION_SYMBOL);
  printf("For example: %s bridge tmp\n", ProgName);
  printf("\n  Options:\n");
  PrintOptions();
  printf("\n%s   %s   %s\n", ProgName, VersionNumber, LastUpdated);
}


/*-------------------------------------------------------------------*/


static void PrintOperatingInfo(char* TSName,
                               char* InitCBName,
                               char* CBName,
                               char* PAName)
{
  if( Value(QuietLevel) >= 2 )
    {
	printf("\n%s   %s   %s\n", ProgName, VersionNumber, LastUpdated);
    printf("Training Set:      %s\n", TSName);
    if(InitialSolutionGiven) printf("Input codebook:    %s\n", InitCBName);
    printf("Output codebook:   %s\n", CBName);
    if(Value(SavePartition)) printf("Output partition:  %s\n", PAName);
    printf("\n");
    PrintSelectedOptions();
    printf("\n");
    }
}


/*======================  SOLUTION HANDLING  ========================*/


static void Initialize(char*          InitCBName,
                       TRAININGSET*   TS,
                       CODEBOOK*      CB,
                       PARTITIONING*  P)
{
/* Initialize data structures and check parameters. */

  if( Value(MaxBookSize)==0 )
      {
      SetValue(MaxBookSize, Value(MinBookSize) );
      }
  if( BookSize(TS) < Value(MaxBookSize) )
      {
      ErrorMessage("ERROR: Training set size (%i) < requested codebook size (%i).\n", Value(MaxBookSize),BookSize(TS));
      }
  if( Value(MaxBookSize) < Value(MinBookSize) )
      {
      ErrorMessage("ERROR: MaxBookSize (%i) < MinBookSize (%i).\n", Value(MaxBookSize), Value(MinBookSize));
      }

  /* Currently initial solution can only be CODEBOOK. It should be
     possible to input also partition as input solution. */

  if( InitialSolutionGiven )
	 {
     ReadCodebook(InitCBName, CB);
	 CreateNewPartitioning(P, TS, BookSize(CB));
	 GenerateOptimalPartitioning(TS, CB, P);
	 }
  else
	 {
     CreateNewCodebook(CB, Value(MaxBookSize), TS);
     /* SetVersionString(CB, CBFILE); */
     CreateNewPartitioning(P, TS, Value(MaxBookSize));
	 }

  SetLSParameters(Value(QuietLevel),
                  Value(SwappingMethod),
                  Value(GLAIterations),
                  Value(LSIterations),
                  Value(MinBookSize),
                  Value(MaxBookSize),
                  MSE);
}


/*-------------------------------------------------------------------*/


static void SaveSolution(char*         CBName,
                         char*         PAName,
                         TRAININGSET*  TS,
                         CODEBOOK*     CB,
                         PARTITIONING* P)
{
  int ndups;

  ndups = DuplicatesInCodebook(CB);
  if( ndups && Value(QuietLevel)>=2 )
     {
	 PrintMessage("WARNING: Final CB contains %i duplicates.\n", ndups);
     }
  SortCodebook(CB, DATA_DESCENDING);
  WriteCodebook(CBName, CB, Value(OverWrite));
  if(Value(SavePartition))
     {
     WritePartitioning(PAName, P, TS, Value(OverWrite));
     }
}


/*===========================  M A I N  ==============================*/


int main(int argc, char* argv[])
{
  TRAININGSET   TS;
  CODEBOOK      CB;
  PARTITIONING  P;
  char			TSName[MAXFILENAME] 	= { '\0' };
  char			CBName[MAXFILENAME] 	= { '\0' };
  char			InitCBName[MAXFILENAME] = { '\0' };
  char			PAName[MAXFILENAME] 	= { '\0' };
  double        Error;

  ParameterInfo paraminfo[3] = { { TSName,     FormatNameTS, 0, INFILE  },
                                 { InitCBName, FormatNameCB, 1, INFILE  },
								 { CBName,	   FormatNameCB, 0, OUTFILE } };

  ParseParameters(argc, argv, 3, paraminfo);
  PickFileName(CBName, PAName);
  CheckFileName(PAName, FormatNamePA);
  PrintOperatingInfo(TSName, InitCBName, CBName, PAName);
  ReadTrainingSet(TSName, &TS);
  Initialize(InitCBName, &TS, &CB, &P);

  IterateByLocalSearch(&TS, &CB, &P, InitialSolutionGiven, &Error);

  SaveSolution(CBName, PAName, &TS, &CB, &P);
  FreeCodebook(&TS);
  FreeCodebook(&CB);
  FreePartitioning(&P);
  checkmemory();
  return( 0 );
}

